/****************************************************************************
 * This file is part of the project AqHome.
 * AqHome (c) by 2023 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include <aqhome/api.h>
#include <aqhome/aqhome.h>

#include "./server.h"

#include <gwenhywfar/gwenhywfar.h>
#include <gwenhywfar/logger.h>
#include <gwenhywfar/cgui.h>
#include <gwenhywfar/debug.h>

#ifdef HAVE_SIGNAL_H
# include <signal.h>
#endif



/* ------------------------------------------------------------------------------------------------
 * defines
 * ------------------------------------------------------------------------------------------------
 */

#define I18N(msg) msg
#define I18S(msg) msg

#define CONNCLEAN_INTERVAL_IN_SECS  2
#define CONNCHECK_INTERVAL_IN_SECS  10
#define HEARTBEAT_INTERVAL_IN_SECS  120 /* every 2mins */



/* ------------------------------------------------------------------------------------------------
 * forward declarations
 * ------------------------------------------------------------------------------------------------
 */

static void _runService(AQH_OBJECT *aqh, AQH_EVENT_LOOP *eventLoop);
#ifdef HAVE_SIGNAL_H
static int _setSignalHandlers(void);
static int _setupSigAction(struct sigaction *sa, int sig);
static void _signalHandler(int s);
#endif
static int _diffInSeconds(time_t t1, time_t t0);



/* ------------------------------------------------------------------------------------------------
 * static vars
 * ------------------------------------------------------------------------------------------------
 */

#ifdef HAVE_SIGNAL_H
static struct sigaction saINT,saTERM, saHUP, saTSTP, saCONT, saPIPE;
#endif

static int stopService=0;



/* ------------------------------------------------------------------------------------------------
 * implementations
 * ------------------------------------------------------------------------------------------------
 */

int main(int argc, char **argv)
{
  int rv;
  AQH_EVENT_LOOP *eventLoop;
  AQH_OBJECT *aqh;
  GWEN_GUI *gui;

  rv=GWEN_Init();
  if (rv) {
    fprintf(stderr, "ERROR: Unable to init Gwen.\n");
    return 2;
  }

  GWEN_Logger_Open(0, "aqhome-nodes", 0, GWEN_LoggerType_Console, GWEN_LoggerFacility_User);
  GWEN_Logger_SetLevel(0, GWEN_LoggerLevel_Warning);

  rv=_setSignalHandlers();
  if (rv<0) {
    DBG_INFO(NULL, "here (%d)", rv);
    return rv;
  }

  rv=AQH_Init();
  if (rv<0) {
    DBG_INFO(NULL, "here (%d)", rv);
    return 2;
  }

  gui=GWEN_Gui_CGui_new();
  GWEN_Gui_SetGui(gui);

  eventLoop=AQH_EventLoop_new();
  aqh=AQH_NodeServer_new(eventLoop);
  rv=AQH_NodeServer_Init(aqh, argc, argv);
  if (rv<0) {
    if (rv==GWEN_ERROR_CLOSE)
      return 1;
    DBG_INFO(NULL, "here (%d)", rv);
    return 2;
  }

  _runService(aqh, eventLoop);

  AQH_NodeServer_Fini(aqh);
  AQH_Object_free(aqh);
  AQH_EventLoop_free(eventLoop);

  GWEN_Gui_SetGui(NULL);
  GWEN_Gui_free(gui);

  return 0;
}



void _runService(AQH_OBJECT *aqh, AQH_EVENT_LOOP *eventLoop)
{
  time_t timeStart;
  int timeout;
  time_t timeLastConnectionCleanup;
  time_t timeLastConnCheck;
  time_t timeLastHeartbeat;

  timeout=AQH_NodeServer_GetTimeout(aqh);
  timeStart=time(NULL);
  timeLastConnectionCleanup=timeStart;
  timeLastConnCheck=timeStart;
  timeLastHeartbeat=timeStart;

  while(!stopService) {
    time_t now;

    AQH_EventLoop_Run(eventLoop, 2000);
    AQH_NodeServer_HandleTtyMsgs(aqh);
    AQH_NodeServer_HandleClientMsgs(aqh);
    AQH_NodeServer_HandleBrokerMsgs(aqh);

    now=time(NULL);

    if (_diffInSeconds(now, timeLastConnectionCleanup)>CONNCLEAN_INTERVAL_IN_SECS) {
      DBG_INFO(NULL, "Cleanup connections");
      AQH_NodeServer_CleanupClients(aqh);
      timeLastConnectionCleanup=now;
    }

    if (_diffInSeconds(now, timeLastConnCheck)>CONNCHECK_INTERVAL_IN_SECS) {
      DBG_INFO(NULL, "Check connections");
      AQH_NodeServer_CheckBrokerConnection(aqh);
      AQH_NodeServer_CheckTtyConnection(aqh);
      timeLastConnCheck=now;
    }

    if (_diffInSeconds(now, timeLastHeartbeat)>HEARTBEAT_INTERVAL_IN_SECS) {
      DBG_INFO(NULL, "Sending heartbeat message");
      AQH_NodeServer_SendHeartbeat(aqh);
      timeLastHeartbeat=now;
    }


    if (timeout && (_diffInSeconds(now, timeStart)>timeout)) {
      DBG_INFO(NULL, "Timeout");
      break;
    }
  } /* while */
}



int _setSignalHandlers(void)
{
#ifdef HAVE_SIGNAL_H
  int rv;

  rv=_setupSigAction(&saINT, SIGINT);
  if (rv)
    return rv;

  rv=_setupSigAction(&saTERM, SIGTERM);
  if (rv)
    return rv;

  rv=_setupSigAction(&saHUP, SIGHUP);
  if (rv)
    return rv;

  rv=_setupSigAction(&saPIPE, SIGPIPE);
  if (rv)
    return rv;

#  ifdef SIGTSTP
  rv=_setupSigAction(&saTSTP, SIGTSTP);
  if (rv)
    return rv;
#  endif

#  ifdef SIGCONT
  rv=_setupSigAction(&saCONT, SIGCONT);
  if (rv)
    return rv;
#  endif
#endif
  return 0;
}



int _setupSigAction(struct sigaction *sa, int sig)
{
  sa->sa_handler=_signalHandler;
  sigemptyset(&sa->sa_mask);
  sa->sa_flags=0;
  if (sigaction(sig, sa, 0)) {
    DBG_ERROR(NULL, "Could not setup signal handler for signal %d", sig);
    return GWEN_ERROR_IO;
  }

  return 0;
}



void _signalHandler(int s)
{
  switch(s) {
  case SIGINT:
  case SIGTERM:
  case SIGHUP:
    DBG_WARN(0, "Received signal %d, stopping service in next loop.",s);
    stopService=1;
    break;
  case SIGPIPE:
    DBG_WARN(0, "Received PIPE signal");
    break;
  default:
    DBG_WARN(0, "Unknown signal %d",s);
    break;
  }
}



int _diffInSeconds(time_t t1, time_t t0)
{
  return t1-t0;
}


